telegram_not_logged_in_text = (
    "*You're not logged  in!*\n"
    "To access this section you should login first\n"
    "It's probably possible by \login command"
)

telegram_permission_denied_text = (
    "*Permission Denied!*\n"
    "You don't have access for this section yet\n"
    "make sure you have the correct access and for more contact admin"
)

telegram_internal_error_text = (
    "*Internal Error*\n"
    "We have encountered an internal error!\n"
    "Our team is notified and is working on this problem.\n"
    "Sorry for the inconvenience."
)

telegram_parse_mode = "Markdown"
